package com.galaxis.manatee.dao;

import com.galaxis.capsule.util.GalaxisRepository;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunWeekMemberHourDO;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.jpa.repository.Query;
import org.springframework.data.repository.query.Param;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * @author kxuan
 */
@Repository
public interface ChuanyunWeekMemberHourDao extends GalaxisRepository<ChuanyunWeekMemberHourDO, String> {

    /**
     * 人员-项目-周度工时汇总
     * 作废
     * @return
     */
    @Query(value = "SELECT chuanyun_self_work_hour.object_id," +
            "DATE_FORMAT(day_log_date,'%Y') as year," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date) AS week," +
            "project_id,user_id," +
            "SUM(standard_work_hour)AS standard_work_hour," +
            "project_type," +
            "`status`" +
            "FROM chuanyun_self_work_hour " +
            "WHERE project_id !='' and LENGTH(user_id) = CHAR_LENGTH(user_id)" +
            "GROUP BY chuanyun_self_work_hour.project_id," +
            "chuanyun_self_work_hour.project_type," +
            "chuanyun_self_work_hour.user_id," +
            "DATE_FORMAT(day_log_date,'%Y')," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)," +
            "chuanyun_self_work_hour.`status`", nativeQuery = true)
    @Deprecated
    List<ChuanyunWeekMemberHourDO> getWeekMemberHour();

    /**
     * 获取最近一个月周工时
     *
     * @return
     */
    @Query(value = "SELECT chuanyun_self_work_hour.object_id, " +
            "LEFT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),4)as year, " +
            "RIGHT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),2) AS week, " +
            "project_id,user_id, " +
            "SUM(standard_work_hour)AS standard_work_hour, " +
            "project_type, " +
            "`status` " +
            "FROM chuanyun_self_work_hour  " +
            "WHERE project_id !=''  " +
            "AND YEARWEEK(chuanyun_self_work_hour.day_log_date,1)>=(YEARWEEK(NOW(),1)-4) " +
            "GROUP BY chuanyun_self_work_hour.project_id, " +
            "chuanyun_self_work_hour.project_type, " +
            "chuanyun_self_work_hour.user_id, " +
            "LEFT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),4), " +
            "RIGHT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),2), " +
            "chuanyun_self_work_hour.`status`", nativeQuery = true)
    List<ChuanyunWeekMemberHourDO> getWeekMemberHourRecently();

    /**
     * 根据用户Id获取周工时列表
     *
     * @param userId 用户ID
     * @return 周工时列表
     */
    @Query(value = "SELECT chuanyun_self_work_hour.object_id, " +
            "LEFT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),4)as year, " +
            "RIGHT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),2) AS week, " +
            "project_id,user_id, " +
            "SUM(standard_work_hour)AS standard_work_hour, " +
            "project_type, " +
            "`status` " +
            "FROM chuanyun_self_work_hour  " +
            "WHERE project_id !='' and user_id=?1 " +
            "GROUP BY chuanyun_self_work_hour.project_id, " +
            "chuanyun_self_work_hour.project_type, " +
            "chuanyun_self_work_hour.user_id, " +
            "LEFT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),4), " +
            "RIGHT(YEARWEEK(chuanyun_self_work_hour.day_log_date,1),2), " +
            "chuanyun_self_work_hour.`status`", nativeQuery = true)
    List<ChuanyunWeekMemberHourDO> getWeekMemberHourByUserId(@Param("userId") String userId);

    /**
     * 获取最近一个月周工时
     *
     * @param pageRequest 分页查询条件
     * @return
     */
    @Query(value = "SELECT * FROM chuanyun_week_member_hour WHERE 1=1 AND CONCAT(chuanyun_week_member_hour.`year`,chuanyun_week_member_hour.`week`)>=(YEARWEEK(NOW(),1)-4) ",
            countQuery = "SELECT COUNT(1) FROM chuanyun_week_member_hour WHERE 1=1 AND CONCAT(chuanyun_week_member_hour.`year`,chuanyun_week_member_hour.`week`)>=(YEARWEEK(NOW(),1)-4) ", nativeQuery = true)
    Page<ChuanyunWeekMemberHourDO> getRecentlyWeekHour(PageRequest pageRequest);

    /**
     * 判断是否有流程中数据
     *
     * @return
     */
    @Query(value = "SELECT chuanyun_self_work_hour.object_id," +
            "DATE_FORMAT(day_log_date,'%Y') as year," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date) AS week," +
            "project_id,user_id," +
            "SUM(standard_work_hour)AS standard_work_hour," +
            "project_type," +
            "`status`" +
            "FROM chuanyun_self_work_hour " +
            "WHERE project_id=?1 and user_id=?2 and DATE_FORMAT(day_log_date,'%Y')=?3 and " +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)=?4 and status=2 and project_type=?5" +
            " GROUP BY chuanyun_self_work_hour.project_id," +
            "chuanyun_self_work_hour.project_type," +
            "chuanyun_self_work_hour.user_id," +
            "DATE_FORMAT(day_log_date,'%Y')," +
            "WEEKOFYEAR(chuanyun_self_work_hour.day_log_date)," +
            "chuanyun_self_work_hour.`status`", nativeQuery = true)
    ChuanyunWeekMemberHourDO getProcessWeekHour(@Param("projectId") String projectId, @Param("userId") String userId, @Param("year") String year, @Param("week") String week, @Param("projectType") String projectType);

    /**
     * 查询项目人员周度是否存在
     *
     * @param projectId   项目Id
     * @param userId      用户Id
     * @param week        周数
     * @param projectType 项目类型
     * @param status      状态
     * @param year        年份
     * @return 周工时记录
     */
    ChuanyunWeekMemberHourDO findByProjectIdAndUserIdAndWeekAndStatusAndProjectTypeAndYear(String projectId, String userId, String week, String status, String projectType, String year);

    /**
     * 删除状态为审核中的数据
     *
     * @param status      状态
     * @param year        年份
     * @param projectType 项目类型
     * @param userId      用户Id
     * @param projectId   项目ID
     * @param week        周数
     */
    @Transactional(rollbackFor = Exception.class)
    void deleteByProjectIdAndUserIdAndYearAndWeekAndStatusAndProjectType(String projectId, String userId, String year, String week, String status, String projectType);

    /**
     * 根据用户ID查询周工时
     * @param userId
     * @return
     */
    List<ChuanyunWeekMemberHourDO> findByUserId(String userId);

}
