package com.galaxis.manatee.task;

import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.ChuanyunSelfWorkHourDao;
import com.galaxis.manatee.dao.ChuanyunUserCompanyDao;
import com.galaxis.manatee.dao.ChuanyunWorkHourDao;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunSelfWorkHourDO;
import com.galaxis.manatee.entity.chuanyun.data.object.ChuanyunWorkHourDO;
import com.galaxis.manatee.service.LogService;
import com.galaxis.manatee.service.LogStandardService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.List;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/8/9 11:02 下午
 */
@Service
@Slf4j
public class WorkHourStatistics {
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunWorkHourDao chuanyunWorkHourDao;
    private final ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao;
    private final LogStandardService logStandardService;
    private final LogService logService;

    public WorkHourStatistics(ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunWorkHourDao chuanyunWorkHourDao, ChuanyunSelfWorkHourDao chuanyunSelfWorkHourDao, LogStandardService logStandardService, LogService logService) {
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunWorkHourDao = chuanyunWorkHourDao;
        this.chuanyunSelfWorkHourDao = chuanyunSelfWorkHourDao;
        this.logStandardService = logStandardService;
        this.logService = logService;
    }

    /**
     * 每10天执行一次所有工时数据的标准化、上传操作。
     */
    @Scheduled(fixedDelay = 864000000L)
    public void updateHour(){
        //每日工时标准化
        standardWorkHour();
        standardWeeklyWorkHour();
        standardMonthlyWorkHour();
        logService.updateAllChuanyunSelfWorkHour();
        logService.updateAllChuanyunWeekMemberHour();
        logService.updateAllChuanyunSelfCost();
    }

    /**
     * 更新所有项目工时
     */
    public void standardHour() {
        //离职人员工时标准化
//        quitStandardHour();
    }

    /**
     * 每日工时标准化
     */
    private void standardWorkHour() {
        chuanyunUserCompanyDao.findAll().forEach(chuanyunUserCompanyDO -> logStandardService.standardSelfWorkHourByUserId(chuanyunUserCompanyDO.getUserId()));
    }

    /**
     * 人员-项目-周度工时汇总
     */
    private void standardWeeklyWorkHour() {
        logService.standardAllWeekMemberHour();
    }

    /**
     * 单月工时汇总
     */
    private void standardMonthlyWorkHour() {
        logService.standardAllSelfCost();
    }

    /**
     * 离职人员工时标准化
     */
    private void quitStandardHour() {
        List<ChuanyunWorkHourDO> selfList = chuanyunWorkHourDao.getQuitSelfWorkHour();
        selfList.forEach(self -> {
            BigDecimal standard = new BigDecimal(8);
            if (self != null) {
                ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO = new ChuanyunSelfWorkHourDO();
                chuanyunSelfWorkHourDO.setStatus(self.getStatus());
                chuanyunSelfWorkHourDO.setUserId(self.getUserId());
                chuanyunSelfWorkHourDO.setProjectType(self.getProjectType());
                if (ChuanYunConstant.PRODUCTION_PROJECT.equals(self.getProjectType())) {
                    chuanyunSelfWorkHourDO.setProjectId(ChuanYunConstant.PRODUCTION_PROJECT_ID);
                } else {
                    chuanyunSelfWorkHourDO.setProjectId(self.getProjectId());
                }
                chuanyunSelfWorkHourDO.setDayLogDate(self.getDayLogDate());
                ChuanyunWorkHourDO chuanyunWorkHourDO = chuanyunWorkHourDao.getQuitDayWorkHour(self.getUserId(), self.getDayLogDate());
                if (chuanyunWorkHourDO != null) {
                    if (chuanyunWorkHourDO.getHoursWorked().compareTo(standard) <= 0) {
                        chuanyunSelfWorkHourDO.setStandardWorkHour(self.getHoursWorked());
                    } else {
                        BigDecimal personDays = (self.getHoursWorked().divide(chuanyunWorkHourDO.getHoursWorked(), 2, RoundingMode.HALF_UP)).multiply(standard);
                        chuanyunSelfWorkHourDO.setStandardWorkHour(personDays);
                    }
                    ChuanyunSelfWorkHourDO chuanyunSelfWorkHourDO1 = chuanyunSelfWorkHourDao.findByUserIdAndAndProjectIdAndAndDayLogDateAndProjectType(self.getUserId(), ChuanYunConstant.PRODUCTION_PROJECT.equals(self.getProjectType()) ? ChuanYunConstant.PRODUCTION_PROJECT_ID : self.getProjectId(), self.getDayLogDate(), self.getProjectType());
                    if (chuanyunSelfWorkHourDO1 != null) {
                        chuanyunSelfWorkHourDao.delete(chuanyunSelfWorkHourDO1);
                    }
                    chuanyunSelfWorkHourDao.save(chuanyunSelfWorkHourDO);
//                    log.info(chuanyunSelfWorkHourDO.getUserId()+"工时标准化");
                }
            }
        });
        log.info("离职人员工时标准化完成" + selfList.size());
    }

}
